@@ -12,7 +12,7 @@
 # language governing permissions and limitations under the License.
 
 import math
-from typing import Sequence
+from typing import Optional, Sequence
 
 from braket.circuits.gate import Gate
 
@@ -22,11 +22,11 @@ class AngledGate(Gate):
     Class `AngledGate` represents a quantum gate that operates on N qubits and an angle.
     """
 
-    def __init__(self, angle: float, qubit_count: int, ascii_symbols: Sequence[str]):
+    def __init__(self, angle: float, qubit_count: Optional[int], ascii_symbols: Sequence[str]):
         """
         Args:
             angle (float): The angle of the gate in radians.
-            qubit_count (int): The number of qubits that this gate interacts with.
+            qubit_count (int, optional): The number of qubits that this gate interacts with.
             ascii_symbols (Sequence[str]): ASCII string symbols for the gate. These are used when
                 printing a diagram of a circuit. The length must be the same as `qubit_count`, and
                 index ordering is expected to correlate with the target ordering on the instruction.
