@@ -43,7 +43,7 @@ class H(Gate):
     """Hadamard gate."""
 
     def __init__(self):
-        super().__init__(qubit_count=1, ascii_symbols=["H"])
+        super().__init__(qubit_count=None, ascii_symbols=["H"])
 
     def to_ir(self, target: QubitSet):
         return ir.H.construct(target=target[0])
@@ -51,6 +51,10 @@ class H(Gate):
     def to_matrix(self) -> np.ndarray:
         return 1.0 / np.sqrt(2.0) * np.array([[1.0, 1.0], [1.0, -1.0]], dtype=complex)
 
+    @staticmethod
+    def fixed_qubit_count() -> int:
+        return 1
+
     @staticmethod
     @circuit.subroutine(register=True)
     def h(target: QubitSetInput) -> Iterable[Instruction]:
@@ -66,7 +70,7 @@ class H(Gate):
             >>> circ = Circuit().h(0)
             >>> circ = Circuit().h([0, 1, 2])
         """
-        return [Instruction(Gate.H(), target=qubit) for qubit in QubitSet(target)]
+        return [Instruction(H(), target=qubit) for qubit in QubitSet(target)]
 
 
 Gate.register_gate(H)
@@ -76,7 +80,7 @@ class I(Gate):  # noqa: E742, E261
     """Identity gate."""
 
     def __init__(self):
-        super().__init__(qubit_count=1, ascii_symbols=["I"])
+        super().__init__(qubit_count=None, ascii_symbols=["I"])
 
     def to_ir(self, target: QubitSet):
         return ir.I.construct(target=target[0])
@@ -84,6 +88,10 @@ class I(Gate):  # noqa: E742, E261
     def to_matrix(self) -> np.ndarray:
         return np.eye(2, dtype=complex)
 
+    @staticmethod
+    def fixed_qubit_count() -> int:
+        return 1
+
     @staticmethod
     @circuit.subroutine(register=True)
     def i(target: QubitSetInput) -> Iterable[Instruction]:
@@ -99,7 +107,7 @@ class I(Gate):  # noqa: E742, E261
             >>> circ = Circuit().i(0)
             >>> circ = Circuit().i([0, 1, 2])
         """
-        return [Instruction(Gate.I(), target=qubit) for qubit in QubitSet(target)]
+        return [Instruction(I(), target=qubit) for qubit in QubitSet(target)]
 
 
 Gate.register_gate(I)
@@ -109,7 +117,7 @@ class X(Gate):
     """Pauli-X gate."""
 
     def __init__(self):
-        super().__init__(qubit_count=1, ascii_symbols=["X"])
+        super().__init__(qubit_count=None, ascii_symbols=["X"])
 
     def to_ir(self, target: QubitSet):
         return ir.X.construct(target=target[0])
@@ -117,6 +125,10 @@ class X(Gate):
     def to_matrix(self) -> np.ndarray:
         return np.array([[0.0, 1.0], [1.0, 0.0]], dtype=complex)
 
+    @staticmethod
+    def fixed_qubit_count() -> int:
+        return 1
+
     @staticmethod
     @circuit.subroutine(register=True)
     def x(target: QubitSetInput) -> Iterable[Instruction]:
@@ -132,7 +144,7 @@ class X(Gate):
             >>> circ = Circuit().x(0)
             >>> circ = Circuit().x([0, 1, 2])
         """
-        return [Instruction(Gate.X(), target=qubit) for qubit in QubitSet(target)]
+        return [Instruction(X(), target=qubit) for qubit in QubitSet(target)]
 
 
 Gate.register_gate(X)
@@ -142,7 +154,7 @@ class Y(Gate):
     """Pauli-Y gate."""
 
     def __init__(self):
-        super().__init__(qubit_count=1, ascii_symbols=["Y"])
+        super().__init__(qubit_count=None, ascii_symbols=["Y"])
 
     def to_ir(self, target: QubitSet):
         return ir.Y.construct(target=target[0])
@@ -150,6 +162,10 @@ class Y(Gate):
     def to_matrix(self) -> np.ndarray:
         return np.array([[0.0, -1.0j], [1.0j, 0.0]], dtype=complex)
 
+    @staticmethod
+    def fixed_qubit_count() -> int:
+        return 1
+
     @staticmethod
     @circuit.subroutine(register=True)
     def y(target: QubitSetInput) -> Iterable[Instruction]:
@@ -165,7 +181,7 @@ class Y(Gate):
             >>> circ = Circuit().y(0)
             >>> circ = Circuit().y([0, 1, 2])
         """
-        return [Instruction(Gate.Y(), target=qubit) for qubit in QubitSet(target)]
+        return [Instruction(Y(), target=qubit) for qubit in QubitSet(target)]
 
 
 Gate.register_gate(Y)
@@ -175,7 +191,7 @@ class Z(Gate):
     """Pauli-Z gate."""
 
     def __init__(self):
-        super().__init__(qubit_count=1, ascii_symbols=["Z"])
+        super().__init__(qubit_count=None, ascii_symbols=["Z"])
 
     def to_ir(self, target: QubitSet):
         return ir.Z.construct(target=target[0])
@@ -183,6 +199,10 @@ class Z(Gate):
     def to_matrix(self) -> np.ndarray:
         return np.array([[1.0, 0.0], [0.0, -1.0]], dtype=complex)
 
+    @staticmethod
+    def fixed_qubit_count() -> int:
+        return 1
+
     @staticmethod
     @circuit.subroutine(register=True)
     def z(target: QubitSetInput) -> Iterable[Instruction]:
@@ -198,7 +218,7 @@ class Z(Gate):
             >>> circ = Circuit().z(0)
             >>> circ = Circuit().z([0, 1, 2])
         """
-        return [Instruction(Gate.Z(), target=qubit) for qubit in QubitSet(target)]
+        return [Instruction(Z(), target=qubit) for qubit in QubitSet(target)]
 
 
 Gate.register_gate(Z)
@@ -208,7 +228,7 @@ class S(Gate):
     """S gate."""
 
     def __init__(self):
-        super().__init__(qubit_count=1, ascii_symbols=["S"])
+        super().__init__(qubit_count=None, ascii_symbols=["S"])
 
     def to_ir(self, target: QubitSet):
         return ir.S.construct(target=target[0])
@@ -217,6 +237,10 @@ class S(Gate):
 
         return np.array([[1.0, 0.0], [0.0, 1.0j]], dtype=complex)
 
+    @staticmethod
+    def fixed_qubit_count() -> int:
+        return 1
+
     @staticmethod
     @circuit.subroutine(register=True)
     def s(target: QubitSetInput) -> Iterable[Instruction]:
@@ -232,7 +256,7 @@ class S(Gate):
             >>> circ = Circuit().s(0)
             >>> circ = Circuit().s([0, 1, 2])
         """
-        return [Instruction(Gate.S(), target=qubit) for qubit in QubitSet(target)]
+        return [Instruction(S(), target=qubit) for qubit in QubitSet(target)]
 
 
 Gate.register_gate(S)
@@ -242,7 +266,7 @@ class Si(Gate):
     """Conjugate transpose of S gate."""
 
     def __init__(self):
-        super().__init__(qubit_count=1, ascii_symbols=["Si"])
+        super().__init__(qubit_count=None, ascii_symbols=["Si"])
 
     def to_ir(self, target: QubitSet):
         return ir.Si.construct(target=target[0])
@@ -250,6 +274,10 @@ class Si(Gate):
     def to_matrix(self) -> np.ndarray:
         return np.array([[1, 0], [0, -1j]], dtype=complex)
 
+    @staticmethod
+    def fixed_qubit_count() -> int:
+        return 1
+
     @staticmethod
     @circuit.subroutine(register=True)
     def si(target: QubitSetInput) -> Iterable[Instruction]:
@@ -265,7 +293,7 @@ class Si(Gate):
             >>> circ = Circuit().si(0)
             >>> circ = Circuit().si([0, 1, 2])
         """
-        return [Instruction(Gate.Si(), target=qubit) for qubit in QubitSet(target)]
+        return [Instruction(Si(), target=qubit) for qubit in QubitSet(target)]
 
 
 Gate.register_gate(Si)
@@ -275,7 +303,7 @@ class T(Gate):
     """T gate."""
 
     def __init__(self):
-        super().__init__(qubit_count=1, ascii_symbols=["T"])
+        super().__init__(qubit_count=None, ascii_symbols=["T"])
 
     def to_ir(self, target: QubitSet):
         return ir.T.construct(target=target[0])
@@ -283,6 +311,10 @@ class T(Gate):
     def to_matrix(self) -> np.ndarray:
         return np.array([[1.0, 0.0], [0.0, np.exp(1j * np.pi / 4)]], dtype=complex)
 
+    @staticmethod
+    def fixed_qubit_count() -> int:
+        return 1
+
     @staticmethod
     @circuit.subroutine(register=True)
     def t(target: QubitSetInput) -> Iterable[Instruction]:
@@ -298,7 +330,7 @@ class T(Gate):
             >>> circ = Circuit().t(0)
             >>> circ = Circuit().t([0, 1, 2])
         """
-        return [Instruction(Gate.T(), target=qubit) for qubit in QubitSet(target)]
+        return [Instruction(T(), target=qubit) for qubit in QubitSet(target)]
 
 
 Gate.register_gate(T)
@@ -308,7 +340,7 @@ class Ti(Gate):
     """Conjugate transpose of T gate."""
 
     def __init__(self):
-        super().__init__(qubit_count=1, ascii_symbols=["Ti"])
+        super().__init__(qubit_count=None, ascii_symbols=["Ti"])
 
     def to_ir(self, target: QubitSet):
         return ir.Ti.construct(target=target[0])
@@ -316,6 +348,10 @@ class Ti(Gate):
     def to_matrix(self) -> np.ndarray:
         return np.array([[1.0, 0.0], [0.0, np.exp(-1j * np.pi / 4)]], dtype=complex)
 
+    @staticmethod
+    def fixed_qubit_count() -> int:
+        return 1
+
     @staticmethod
     @circuit.subroutine(register=True)
     def ti(target: QubitSetInput) -> Iterable[Instruction]:
@@ -331,7 +367,7 @@ class Ti(Gate):
             >>> circ = Circuit().ti(0)
             >>> circ = Circuit().ti([0, 1, 2])
         """
-        return [Instruction(Gate.Ti(), target=qubit) for qubit in QubitSet(target)]
+        return [Instruction(Ti(), target=qubit) for qubit in QubitSet(target)]
 
 
 Gate.register_gate(Ti)
@@ -341,7 +377,7 @@ class V(Gate):
     """Square root of not gate."""
 
     def __init__(self):
-        super().__init__(qubit_count=1, ascii_symbols=["V"])
+        super().__init__(qubit_count=None, ascii_symbols=["V"])
 
     def to_ir(self, target: QubitSet):
         return ir.V.construct(target=target[0])
@@ -349,6 +385,10 @@ class V(Gate):
     def to_matrix(self) -> np.ndarray:
         return np.array([[0.5 + 0.5j, 0.5 - 0.5j], [0.5 - 0.5j, 0.5 + 0.5j]], dtype=complex)
 
+    @staticmethod
+    def fixed_qubit_count() -> int:
+        return 1
+
     @staticmethod
     @circuit.subroutine(register=True)
     def v(target: QubitSetInput) -> Iterable[Instruction]:
@@ -364,7 +404,7 @@ class V(Gate):
             >>> circ = Circuit().v(0)
             >>> circ = Circuit().v([0, 1, 2])
         """
-        return [Instruction(Gate.V(), target=qubit) for qubit in QubitSet(target)]
+        return [Instruction(V(), target=qubit) for qubit in QubitSet(target)]
 
 
 Gate.register_gate(V)
@@ -374,7 +414,7 @@ class Vi(Gate):
     """Conjugate transpose of square root of not gate."""
 
     def __init__(self):
-        super().__init__(qubit_count=1, ascii_symbols=["Vi"])
+        super().__init__(qubit_count=None, ascii_symbols=["Vi"])
 
     def to_ir(self, target: QubitSet):
         return ir.Vi.construct(target=target[0])
@@ -382,6 +422,10 @@ class Vi(Gate):
     def to_matrix(self) -> np.ndarray:
         return np.array(([[0.5 - 0.5j, 0.5 + 0.5j], [0.5 + 0.5j, 0.5 - 0.5j]]), dtype=complex)
 
+    @staticmethod
+    def fixed_qubit_count() -> int:
+        return 1
+
     @staticmethod
     @circuit.subroutine(register=True)
     def vi(target: QubitSetInput) -> Iterable[Instruction]:
@@ -397,7 +441,7 @@ class Vi(Gate):
             >>> circ = Circuit().vi(0)
             >>> circ = Circuit().vi([0, 1, 2])
         """
-        return [Instruction(Gate.Vi(), target=qubit) for qubit in QubitSet(target)]
+        return [Instruction(Vi(), target=qubit) for qubit in QubitSet(target)]
 
 
 Gate.register_gate(Vi)
@@ -414,7 +458,7 @@ class Rx(AngledGate):
     """
 
     def __init__(self, angle: float):
-        super().__init__(angle=angle, qubit_count=1, ascii_symbols=["Rx({:.3g})".format(angle)])
+        super().__init__(angle=angle, qubit_count=None, ascii_symbols=["Rx({:.3g})".format(angle)])
 
     def to_ir(self, target: QubitSet):
         return ir.Rx.construct(target=target[0], angle=self.angle)
@@ -424,9 +468,13 @@ class Rx(AngledGate):
         sin = np.sin(self.angle / 2)
         return np.array([[cos, -1j * sin], [-1j * sin, cos]], dtype=complex)
 
+    @staticmethod
+    def fixed_qubit_count() -> int:
+        return 1
+
     @staticmethod
     @circuit.subroutine(register=True)
-    def rx(target: QubitInput, angle: float) -> Instruction:
+    def rx(target: QubitInput, angle: float) -> Iterable[Instruction]:
         """Registers this function into the circuit class.
 
         Args:
@@ -434,12 +482,12 @@ class Rx(AngledGate):
             angle (float): Angle in radians.
 
         Returns:
-            Instruction: Rx instruction.
+            Iterable[Instruction]: Rx instruction.
 
         Examples:
             >>> circ = Circuit().rx(0, 0.15)
         """
-        return [Instruction(Gate.Rx(angle), target=qubit) for qubit in QubitSet(target)]
+        return [Instruction(Rx(angle), target=qubit) for qubit in QubitSet(target)]
 
 
 Gate.register_gate(Rx)
@@ -453,7 +501,7 @@ class Ry(AngledGate):
     """
 
     def __init__(self, angle: float):
-        super().__init__(angle=angle, qubit_count=1, ascii_symbols=["Ry({:.3g})".format(angle)])
+        super().__init__(angle=angle, qubit_count=None, ascii_symbols=["Ry({:.3g})".format(angle)])
 
     def to_ir(self, target: QubitSet):
         return ir.Ry.construct(target=target[0], angle=self.angle)
@@ -463,9 +511,13 @@ class Ry(AngledGate):
         sin = np.sin(self.angle / 2)
         return np.array([[cos, -sin], [+sin, cos]], dtype=complex)
 
+    @staticmethod
+    def fixed_qubit_count() -> int:
+        return 1
+
     @staticmethod
     @circuit.subroutine(register=True)
-    def ry(target: QubitInput, angle: float) -> Instruction:
+    def ry(target: QubitInput, angle: float) -> Iterable[Instruction]:
         """Registers this function into the circuit class.
 
         Args:
@@ -473,12 +525,12 @@ class Ry(AngledGate):
             angle (float): Angle in radians.
 
         Returns:
-            Instruction: Ry instruction.
+            Iterable[Instruction]: Ry instruction.
 
         Examples:
             >>> circ = Circuit().ry(0, 0.15)
         """
-        return [Instruction(Gate.Ry(angle), target=qubit) for qubit in QubitSet(target)]
+        return [Instruction(Ry(angle), target=qubit) for qubit in QubitSet(target)]
 
 
 Gate.register_gate(Ry)
@@ -492,7 +544,7 @@ class Rz(AngledGate):
     """
 
     def __init__(self, angle: float):
-        super().__init__(angle=angle, qubit_count=1, ascii_symbols=["Rz({:.3g})".format(angle)])
+        super().__init__(angle=angle, qubit_count=None, ascii_symbols=["Rz({:.3g})".format(angle)])
 
     def to_ir(self, target: QubitSet):
         return ir.Rz.construct(target=target[0], angle=self.angle)
@@ -502,9 +554,13 @@ class Rz(AngledGate):
             [[np.exp(-1j * self.angle / 2), 0], [0, np.exp(1j * self.angle / 2)]], dtype=complex
         )
 
+    @staticmethod
+    def fixed_qubit_count() -> int:
+        return 1
+
     @staticmethod
     @circuit.subroutine(register=True)
-    def rz(target: QubitInput, angle: float) -> Instruction:
+    def rz(target: QubitInput, angle: float) -> Iterable[Instruction]:
         """Registers this function into the circuit class.
 
         Args:
@@ -512,12 +568,12 @@ class Rz(AngledGate):
             angle (float): Angle in radians.
 
         Returns:
-            Instruction: Rz instruction.
+            Iterable[Instruction]: Rz instruction.
 
         Examples:
             >>> circ = Circuit().rz(0, 0.15)
         """
-        return [Instruction(Gate.Rz(angle), target=qubit) for qubit in QubitSet(target)]
+        return [Instruction(Rz(angle), target=qubit) for qubit in QubitSet(target)]
 
 
 Gate.register_gate(Rz)
@@ -531,7 +587,9 @@ class PhaseShift(AngledGate):
     """
 
     def __init__(self, angle: float):
-        super().__init__(angle=angle, qubit_count=1, ascii_symbols=["PHASE({:.3g})".format(angle)])
+        super().__init__(
+            angle=angle, qubit_count=None, ascii_symbols=["PHASE({:.3g})".format(angle)]
+        )
 
     def to_ir(self, target: QubitSet):
         return ir.PhaseShift.construct(target=target[0], angle=self.angle)
@@ -539,9 +597,13 @@ class PhaseShift(AngledGate):
     def to_matrix(self) -> np.ndarray:
         return np.array([[1.0, 0.0], [0.0, np.exp(1j * self.angle)]], dtype=complex)
 
+    @staticmethod
+    def fixed_qubit_count() -> int:
+        return 1
+
     @staticmethod
     @circuit.subroutine(register=True)
-    def phaseshift(target: QubitInput, angle: float) -> Instruction:
+    def phaseshift(target: QubitInput, angle: float) -> Iterable[Instruction]:
         """Registers this function into the circuit class.
 
         Args:
@@ -549,12 +611,12 @@ class PhaseShift(AngledGate):
             angle (float): Angle in radians.
 
         Returns:
-            Instruction: PhaseShift instruction.
+            Iterable[Instruction]: PhaseShift instruction.
 
         Examples:
             >>> circ = Circuit().phaseshift(0, 0.15)
         """
-        return [Instruction(Gate.PhaseShift(angle), target=qubit) for qubit in QubitSet(target)]
+        return [Instruction(PhaseShift(angle), target=qubit) for qubit in QubitSet(target)]
 
 
 Gate.register_gate(PhaseShift)
@@ -567,7 +629,7 @@ class CNot(Gate):
     """Controlled NOT gate."""
 
     def __init__(self):
-        super().__init__(qubit_count=2, ascii_symbols=["C", "X"])
+        super().__init__(qubit_count=None, ascii_symbols=["C", "X"])
 
     def to_ir(self, target: QubitSet):
         return ir.CNot.construct(control=target[0], target=target[1])
@@ -583,6 +645,10 @@ class CNot(Gate):
             dtype=complex,
         )
 
+    @staticmethod
+    def fixed_qubit_count() -> int:
+        return 2
+
     @staticmethod
     @circuit.subroutine(register=True)
     def cnot(control: QubitInput, target: QubitInput) -> Instruction:
@@ -598,7 +664,7 @@ class CNot(Gate):
         Examples:
             >>> circ = Circuit().cnot(0, 1)
         """
-        return Instruction(Gate.CNot(), target=[control, target])
+        return Instruction(CNot(), target=[control, target])
 
 
 Gate.register_gate(CNot)
@@ -608,7 +674,7 @@ class Swap(Gate):
     """Swap gate."""
 
     def __init__(self):
-        super().__init__(qubit_count=2, ascii_symbols=["SWAP", "SWAP"])
+        super().__init__(qubit_count=None, ascii_symbols=["SWAP", "SWAP"])
 
     def to_ir(self, target: QubitSet):
         return ir.Swap.construct(targets=[target[0], target[1]])
@@ -624,6 +690,10 @@ class Swap(Gate):
             dtype=complex,
         )
 
+    @staticmethod
+    def fixed_qubit_count() -> int:
+        return 2
+
     @staticmethod
     @circuit.subroutine(register=True)
     def swap(target1: QubitInput, target2: QubitInput) -> Instruction:
@@ -639,7 +709,7 @@ class Swap(Gate):
         Examples:
             >>> circ = Circuit().swap(0, 1)
         """
-        return Instruction(Gate.Swap(), target=[target1, target2])
+        return Instruction(Swap(), target=[target1, target2])
 
 
 Gate.register_gate(Swap)
@@ -649,7 +719,7 @@ class ISwap(Gate):
     """ISwap gate."""
 
     def __init__(self):
-        super().__init__(qubit_count=2, ascii_symbols=["ISWAP", "ISWAP"])
+        super().__init__(qubit_count=None, ascii_symbols=["ISWAP", "ISWAP"])
 
     def to_ir(self, target: QubitSet):
         return ir.ISwap.construct(targets=[target[0], target[1]])
@@ -665,6 +735,10 @@ class ISwap(Gate):
             dtype=complex,
         )
 
+    @staticmethod
+    def fixed_qubit_count() -> int:
+        return 2
+
     @staticmethod
     @circuit.subroutine(register=True)
     def iswap(target1: QubitInput, target2: QubitInput) -> Instruction:
@@ -680,7 +754,7 @@ class ISwap(Gate):
         Examples:
             >>> circ = Circuit().iswap(0, 1)
         """
-        return Instruction(Gate.ISwap(), target=[target1, target2])
+        return Instruction(ISwap(), target=[target1, target2])
 
 
 Gate.register_gate(ISwap)
@@ -696,7 +770,7 @@ class PSwap(AngledGate):
     def __init__(self, angle: float):
         super().__init__(
             angle=angle,
-            qubit_count=2,
+            qubit_count=None,
             ascii_symbols=["PSWAP({:.3g})".format(angle), "PSWAP({:.3g})".format(angle)],
         )
 
@@ -714,6 +788,10 @@ class PSwap(AngledGate):
             dtype=complex,
         )
 
+    @staticmethod
+    def fixed_qubit_count() -> int:
+        return 2
+
     @staticmethod
     @circuit.subroutine(register=True)
     def pswap(target1: QubitInput, target2: QubitInput, angle: float) -> Instruction:
@@ -729,7 +807,7 @@ class PSwap(AngledGate):
         Examples:
             >>> circ = Circuit().pswap(0, 1, 0.15)
         """
-        return Instruction(Gate.PSwap(angle), target=[target1, target2])
+        return Instruction(PSwap(angle), target=[target1, target2])
 
 
 Gate.register_gate(PSwap)
@@ -747,7 +825,7 @@ class XY(AngledGate):
     def __init__(self, angle: float):
         super().__init__(
             angle=angle,
-            qubit_count=2,
+            qubit_count=None,
             ascii_symbols=["XY({:.3g})".format(angle), "XY({:.3g})".format(angle)],
         )
 
@@ -767,6 +845,10 @@ class XY(AngledGate):
             dtype=complex,
         )
 
+    @staticmethod
+    def fixed_qubit_count() -> int:
+        return 2
+
     @staticmethod
     @circuit.subroutine(register=True)
     def xy(target1: QubitInput, target2: QubitInput, angle: float) -> Instruction:
@@ -782,7 +864,7 @@ class XY(AngledGate):
         Examples:
             >>> circ = Circuit().xy(0, 1, 0.15)
         """
-        return Instruction(Gate.XY(angle), target=[target1, target2])
+        return Instruction(XY(angle), target=[target1, target2])
 
 
 Gate.register_gate(XY)
@@ -797,7 +879,7 @@ class CPhaseShift(AngledGate):
 
     def __init__(self, angle: float):
         super().__init__(
-            angle=angle, qubit_count=2, ascii_symbols=["C", "PHASE({:.3g})".format(angle)]
+            angle=angle, qubit_count=None, ascii_symbols=["C", "PHASE({:.3g})".format(angle)]
         )
 
     def to_ir(self, target: QubitSet):
@@ -806,6 +888,10 @@ class CPhaseShift(AngledGate):
     def to_matrix(self) -> np.ndarray:
         return np.diag([1.0, 1.0, 1.0, np.exp(1j * self.angle)])
 
+    @staticmethod
+    def fixed_qubit_count() -> int:
+        return 2
+
     @staticmethod
     @circuit.subroutine(register=True)
     def cphaseshift(control: QubitInput, target: QubitInput, angle: float) -> Instruction:
@@ -822,7 +908,7 @@ class CPhaseShift(AngledGate):
         Examples:
             >>> circ = Circuit().cphaseshift(0, 1, 0.15)
         """
-        return Instruction(Gate.CPhaseShift(angle), target=[control, target])
+        return Instruction(CPhaseShift(angle), target=[control, target])
 
 
 Gate.register_gate(CPhaseShift)
@@ -837,7 +923,7 @@ class CPhaseShift00(AngledGate):
 
     def __init__(self, angle: float):
         super().__init__(
-            angle=angle, qubit_count=2, ascii_symbols=["C", "PHASE00({:.3g})".format(angle)]
+            angle=angle, qubit_count=None, ascii_symbols=["C", "PHASE00({:.3g})".format(angle)]
         )
 
     def to_ir(self, target: QubitSet):
@@ -846,6 +932,10 @@ class CPhaseShift00(AngledGate):
     def to_matrix(self) -> np.ndarray:
         return np.diag([np.exp(1j * self.angle), 1.0, 1.0, 1.0])
 
+    @staticmethod
+    def fixed_qubit_count() -> int:
+        return 2
+
     @staticmethod
     @circuit.subroutine(register=True)
     def cphaseshift00(control: QubitInput, target: QubitInput, angle: float) -> Instruction:
@@ -862,7 +952,7 @@ class CPhaseShift00(AngledGate):
         Examples:
             >>> circ = Circuit().cphaseshift00(0, 1, 0.15)
         """
-        return Instruction(Gate.CPhaseShift00(angle), target=[control, target])
+        return Instruction(CPhaseShift00(angle), target=[control, target])
 
 
 Gate.register_gate(CPhaseShift00)
@@ -877,7 +967,7 @@ class CPhaseShift01(AngledGate):
 
     def __init__(self, angle: float):
         super().__init__(
-            angle=angle, qubit_count=2, ascii_symbols=["C", "PHASE01({:.3g})".format(angle)]
+            angle=angle, qubit_count=None, ascii_symbols=["C", "PHASE01({:.3g})".format(angle)]
         )
 
     def to_ir(self, target: QubitSet):
@@ -886,6 +976,10 @@ class CPhaseShift01(AngledGate):
     def to_matrix(self) -> np.ndarray:
         return np.diag([1.0, np.exp(1j * self.angle), 1.0, 1.0])
 
+    @staticmethod
+    def fixed_qubit_count() -> int:
+        return 2
+
     @staticmethod
     @circuit.subroutine(register=True)
     def cphaseshift01(control: QubitInput, target: QubitInput, angle: float) -> Instruction:
@@ -902,7 +996,7 @@ class CPhaseShift01(AngledGate):
         Examples:
             >>> circ = Circuit().cphaseshift01(0, 1, 0.15)
         """
-        return Instruction(Gate.CPhaseShift01(angle), target=[control, target])
+        return Instruction(CPhaseShift01(angle), target=[control, target])
 
 
 Gate.register_gate(CPhaseShift01)
@@ -917,7 +1011,7 @@ class CPhaseShift10(AngledGate):
 
     def __init__(self, angle: float):
         super().__init__(
-            angle=angle, qubit_count=2, ascii_symbols=["C", "PHASE10({:.3g})".format(angle)]
+            angle=angle, qubit_count=None, ascii_symbols=["C", "PHASE10({:.3g})".format(angle)]
         )
 
     def to_ir(self, target: QubitSet):
@@ -926,6 +1020,10 @@ class CPhaseShift10(AngledGate):
     def to_matrix(self) -> np.ndarray:
         return np.diag([1.0, 1.0, np.exp(1j * self.angle), 1.0])
 
+    @staticmethod
+    def fixed_qubit_count() -> int:
+        return 2
+
     @staticmethod
     @circuit.subroutine(register=True)
     def cphaseshift10(control: QubitInput, target: QubitInput, angle: float) -> Instruction:
@@ -942,7 +1040,7 @@ class CPhaseShift10(AngledGate):
         Examples:
             >>> circ = Circuit().cphaseshift10(0, 1, 0.15)
         """
-        return Instruction(Gate.CPhaseShift10(angle), target=[control, target])
+        return Instruction(CPhaseShift10(angle), target=[control, target])
 
 
 Gate.register_gate(CPhaseShift10)
@@ -952,7 +1050,7 @@ class CY(Gate):
     """Controlled Pauli-Y gate."""
 
     def __init__(self):
-        super().__init__(qubit_count=2, ascii_symbols=["C", "Y"])
+        super().__init__(qubit_count=None, ascii_symbols=["C", "Y"])
 
     def to_ir(self, target: QubitSet):
         return ir.CY.construct(control=target[0], target=target[1])
@@ -968,6 +1066,10 @@ class CY(Gate):
             dtype=complex,
         )
 
+    @staticmethod
+    def fixed_qubit_count() -> int:
+        return 2
+
     @staticmethod
     @circuit.subroutine(register=True)
     def cy(control: QubitInput, target: QubitInput) -> Instruction:
@@ -983,7 +1085,7 @@ class CY(Gate):
         Examples:
             >>> circ = Circuit().cy(0, 1)
         """
-        return Instruction(Gate.CY(), target=[control, target])
+        return Instruction(CY(), target=[control, target])
 
 
 Gate.register_gate(CY)
@@ -993,13 +1095,17 @@ class CZ(Gate):
     """Controlled Pauli-Z gate."""
 
     def __init__(self):
-        super().__init__(qubit_count=2, ascii_symbols=["C", "Z"])
+        super().__init__(qubit_count=None, ascii_symbols=["C", "Z"])
 
     def to_ir(self, target: QubitSet):
         return ir.CZ.construct(control=target[0], target=target[1])
 
     def to_matrix(self) -> np.ndarray:
-        return np.diag([1.0, 1.0, 1.0, -1.0])
+        return np.diag([complex(1.0), 1.0, 1.0, -1.0])
+
+    @staticmethod
+    def fixed_qubit_count() -> int:
+        return 2
 
     @staticmethod
     @circuit.subroutine(register=True)
@@ -1016,7 +1122,7 @@ class CZ(Gate):
         Examples:
             >>> circ = Circuit().cz(0, 1)
         """
-        return Instruction(Gate.CZ(), target=[control, target])
+        return Instruction(CZ(), target=[control, target])
 
 
 Gate.register_gate(CZ)
@@ -1034,7 +1140,7 @@ class XX(AngledGate):
     def __init__(self, angle: float):
         super().__init__(
             angle=angle,
-            qubit_count=2,
+            qubit_count=None,
             ascii_symbols=["XX({:.3g})".format(angle), "XX({:.3g})".format(angle)],
         )
 
@@ -1054,6 +1160,10 @@ class XX(AngledGate):
             dtype=complex,
         )
 
+    @staticmethod
+    def fixed_qubit_count() -> int:
+        return 2
+
     @staticmethod
     @circuit.subroutine(register=True)
     def xx(target1: QubitInput, target2: QubitInput, angle: float) -> Instruction:
@@ -1070,7 +1180,7 @@ class XX(AngledGate):
         Examples:
             >>> circ = Circuit().xx(0, 1, 0.15)
         """
-        return Instruction(Gate.XX(angle), target=[target1, target2])
+        return Instruction(XX(angle), target=[target1, target2])
 
 
 Gate.register_gate(XX)
@@ -1088,7 +1198,7 @@ class YY(AngledGate):
     def __init__(self, angle: float):
         super().__init__(
             angle=angle,
-            qubit_count=2,
+            qubit_count=None,
             ascii_symbols=["YY({:.3g})".format(angle), "YY({:.3g})".format(angle)],
         )
 
@@ -1108,6 +1218,10 @@ class YY(AngledGate):
             dtype=complex,
         )
 
+    @staticmethod
+    def fixed_qubit_count() -> int:
+        return 2
+
     @staticmethod
     @circuit.subroutine(register=True)
     def yy(target1: QubitInput, target2: QubitInput, angle: float) -> Instruction:
@@ -1124,7 +1238,7 @@ class YY(AngledGate):
         Examples:
             >>> circ = Circuit().yy(0, 1, 0.15)
         """
-        return Instruction(Gate.YY(angle), target=[target1, target2])
+        return Instruction(YY(angle), target=[target1, target2])
 
 
 Gate.register_gate(YY)
@@ -1142,7 +1256,7 @@ class ZZ(AngledGate):
     def __init__(self, angle: float):
         super().__init__(
             angle=angle,
-            qubit_count=2,
+            qubit_count=None,
             ascii_symbols=["ZZ({:.3g})".format(angle), "ZZ({:.3g})".format(angle)],
         )
 
@@ -1160,6 +1274,10 @@ class ZZ(AngledGate):
             dtype=complex,
         )
 
+    @staticmethod
+    def fixed_qubit_count() -> int:
+        return 2
+
     @staticmethod
     @circuit.subroutine(register=True)
     def zz(target1: QubitInput, target2: QubitInput, angle: float) -> Instruction:
@@ -1176,7 +1294,7 @@ class ZZ(AngledGate):
         Examples:
             >>> circ = Circuit().zz(0, 1, 0.15)
         """
-        return Instruction(Gate.ZZ(angle), target=[target1, target2])
+        return Instruction(ZZ(angle), target=[target1, target2])
 
 
 Gate.register_gate(ZZ)
@@ -1189,7 +1307,7 @@ class CCNot(Gate):
     """CCNOT gate or Toffoli gate."""
 
     def __init__(self):
-        super().__init__(qubit_count=3, ascii_symbols=["C", "C", "X"])
+        super().__init__(qubit_count=None, ascii_symbols=["C", "C", "X"])
 
     def to_ir(self, target: QubitSet):
         return ir.CCNot.construct(controls=[target[0], target[1]], target=target[2])
@@ -1209,6 +1327,10 @@ class CCNot(Gate):
             dtype=complex,
         )
 
+    @staticmethod
+    def fixed_qubit_count() -> int:
+        return 3
+
     @staticmethod
     @circuit.subroutine(register=True)
     def ccnot(control1: QubitInput, control2: QubitInput, target: QubitInput) -> Instruction:
@@ -1225,7 +1347,7 @@ class CCNot(Gate):
         Examples:
             >>> circ = Circuit().ccnot(0, 1, 2)
         """
-        return Instruction(Gate.CCNot(), target=[control1, control2, target])
+        return Instruction(CCNot(), target=[control1, control2, target])
 
 
 Gate.register_gate(CCNot)
@@ -1235,7 +1357,7 @@ class CSwap(Gate):
     """Controlled Swap gate."""
 
     def __init__(self):
-        super().__init__(qubit_count=3, ascii_symbols=["C", "SWAP", "SWAP"])
+        super().__init__(qubit_count=None, ascii_symbols=["C", "SWAP", "SWAP"])
 
     def to_ir(self, target: QubitSet):
         return ir.CSwap.construct(control=target[0], targets=[target[1], target[2]])
@@ -1255,6 +1377,10 @@ class CSwap(Gate):
             dtype=complex,
         )
 
+    @staticmethod
+    def fixed_qubit_count() -> int:
+        return 3
+
     @staticmethod
     @circuit.subroutine(register=True)
     def cswap(control: QubitInput, target1: QubitInput, target2: QubitInput) -> Instruction:
@@ -1271,7 +1397,7 @@ class CSwap(Gate):
         Examples:
             >>> circ = Circuit().cswap(0, 1, 2)
         """
-        return Instruction(Gate.CSwap(), target=[control, target1, target2])
+        return Instruction(CSwap(), target=[control, target1, target2])
 
 
 Gate.register_gate(CSwap)
@@ -1345,7 +1471,7 @@ class Unitary(Gate):
         if 2 ** len(targets) != matrix.shape[0]:
             raise ValueError("Dimensions of the supplied unitary are incompatible with the targets")
 
-        return Instruction(Gate.Unitary(matrix, display_name), target=targets)
+        return Instruction(Unitary(matrix, display_name), target=targets)
 
 
 Gate.register_gate(Unitary)
